r"""
:mod:`dex` is an index under construction"""

from .library import load_library

__all__ = ["library"]

__author__ = "Louis Maddox"
__license__ = "MIT"
__description__ = "Indexer under construction."
__url__ = "https://github.com/lmmx/dex"
__uri__ = __url__
__email__ = "louismmx@gmail.com"
